@@ -14,9 +14,11 @@ from kupala.asgi import ASGIHandler
 from kupala.config import Config
 from kupala.console.application import ConsoleApplication
 from kupala.contracts import TemplateRenderer
+from kupala.di import Injector
 from kupala.dotenv import DotEnv
 from kupala.extensions import (
     AuthenticationExtension,
+    ConsoleExtension,
     JinjaExtension,
     MailExtension,
     PasswordsExtension,
@@ -35,6 +37,7 @@ from kupala.utils import resolve_path
 
 class Kupala:
     request_class = Request
+    routes_config: typing.Callable[[Routes], None] | str | None = None
 
     def __init__(
         self,
@@ -49,6 +52,7 @@ class Kupala:
         lifespan_handlers: list[typing.Callable[[Kupala], typing.AsyncContextManager[None]]] = None,
         exception_handler: typing.Callable[[Request, Exception], Response] | None = None,
         template_dir: str | list[str] = 'templates',
+        routes_config: typing.Callable[[Routes], None] | str | None = None,
         commands: list[click.Command] = None,
         storages: dict[str, Storage] = None,
         renderer: TemplateRenderer = None,
@@ -57,6 +61,9 @@ class Kupala:
         self.secret_key = secret_key
         self.request_class = request_class or self.request_class or Request
 
+        # routes config preflight
+        self.routes_config = routes_config or self.routes_config
+
         # read environment and set default variables
         self.environ = DotEnv([env_file])
         self.secret_key = secret_key or self.environ.get('SECRET_KEY', default=None, check_file=True)
@@ -86,14 +93,22 @@ class Kupala:
         self.mail = MailExtension()
         self.auth = AuthenticationExtension(self)
         self.storages = StoragesExtension(storages)
-        self.commands = commands or []
+        self.commands = ConsoleExtension(commands)
         self.signer = SignerExtension(self.secret_key)
         self.renderer = RendererExtension(renderer)
+        self.di = Injector(self)
 
         set_current_application(self)
+
+        # bind extensions to this app
         self._initialize()
+
+        # configure extensions
         self.configure()
 
+        # load routes
+        self.apply_routes(self.routes)
+
         # ASGI app instance
         self._asgi_app: ASGIApp | None = None
 
@@ -104,11 +119,18 @@ class Kupala:
     def configure(self) -> None:
         pass
 
-    def apply_middleware(self) -> None:
+    def apply_middleware(self, stack: MiddlewareStack) -> None:
         pass
 
+    def apply_routes(self, routes: Routes) -> None:
+        if self.routes_config:
+            if callable(self.routes_config):
+                self.routes_config(routes)
+            else:
+                routes.include(self.routes_config)
+
     def create_asgi_app(self) -> ASGIApp:
-        self.apply_middleware()
+        self.apply_middleware(self.middleware)
         return ASGIHandler(
             app=self,
             debug=self.debug,
