@@ -1,13 +1,16 @@
 from __future__ import annotations
 
+import click
 import itsdangerous.signer
 import jinja2
 import jinja2.ext
+import logging
 import os
 import typing
+from email.message import Message
 from functools import cached_property
 from imia import BaseAuthenticator, LoginManager, UserProvider, UserToken
-from mailers import Encrypter, Mailer, Plugin, Signer, create_transport_from_url
+from mailers import Email, Encrypter, Mailer, Plugin, SentMessages, Signer, create_transport_from_url
 
 from kupala.contracts import PasswordHasher, TemplateRenderer
 from kupala.di import to_app_injectable, to_request_injectable
@@ -52,6 +55,9 @@ class PasswordsExtension(Extension):
             return import_string(imports[backend])
         return backend
 
+    def initialize(self, app: Kupala) -> None:
+        app.di.prefer_for(PasswordHasher, self._manager)
+
 
 class RendererExtension(Extension):
     def __init__(self, renderer: TemplateRenderer | None = None) -> None:
@@ -109,6 +115,10 @@ class MailExtension(Extension):
     def initialize(self, app: Kupala) -> None:
         to_app_injectable(Mailer, lambda app: self.get_default())
 
+    async def send(self, message: Email | Message, mailer: str = None) -> SentMessages:
+        mailer_name = mailer or self.default
+        return await self.get(mailer_name).send(message)
+
 
 class AuthenticationExtension(Extension):
     def __init__(self, app: Kupala) -> None:
@@ -205,12 +215,15 @@ class JinjaExtension(Extension):
 
     @cached_property
     def loader(self) -> jinja2.BaseLoader:
+        """Create and return instance of jinja2.BaseLoader.
+        If no `loader` constructor argument passed then jinja2.FileSystemLoader will be used."""
         if self._loader:
             return self._loader
         return jinja2.FileSystemLoader(searchpath=[resolve_path(directory) for directory in self.template_dirs])
 
     @cached_property
     def env(self) -> jinja2.Environment:
+        """Create and return jinja2.Environment instance."""
         if self._env:
             return self._env
         env = jinja2.Environment(loader=self.loader, extensions=self.extensions)
@@ -224,10 +237,62 @@ class JinjaExtension(Extension):
 
         return env
 
-    @property
+    @cached_property
     def renderer(self) -> JinjaRenderer:
+        """Create and return Jinja template renderer."""
         return JinjaRenderer(self.env)
 
+    def add_template_dirs(self, directories: list[str]) -> None:
+        """Add additional directories for template search."""
+        if isinstance(self.env.loader, jinja2.FileSystemLoader):
+            self.env.loader.searchpath = [resolve_path(directory) for directory in directories]
+        else:
+            logging.warning('Hot directory update supported only for jinja2.FileSystemLoader loader.')
+
+    def add_globals(self, globals: dict[str, typing.Any]) -> None:
+        """Add global variables to jinja environment."""
+        self.env.globals.update(globals)
+
+    def add_filters(self, filters: dict[str, typing.Callable]) -> None:
+        """Add filters to jinja environment."""
+        self.env.filters.update(filters)
+
+    def add_policies(self, policies: dict[str, typing.Any]) -> None:
+        """Add policies to jinja environment."""
+        self.env.policies.update(policies)
+
+    def add_tests(self, tests: dict[str, typing.Any]) -> None:
+        """Add tests to jinja environment."""
+        self.env.tests.update(tests)
+
+    def add_extensions(self, *extension: str | typing.Type[jinja2.ext.Extension]) -> None:
+        """Add extension to jinja environment."""
+        for _extension in extension:
+            self.env.add_extension(_extension)
+
+    def configure(
+        self,
+        template_dirs: list[str] = None,
+        globals: dict[str, typing.Any] = None,
+        filters: dict[str, typing.Callable] = None,
+        policies: dict[str, typing.Any] = None,
+        tests: dict[str, typing.Any] = None,
+        extensions: list[str | typing.Type[jinja2.ext.Extension]] = None,
+    ) -> None:
+        """Configure multiple options at once."""
+        if template_dirs:
+            self.add_template_dirs(template_dirs)
+        if globals:
+            self.add_globals(globals)
+        if filters:
+            self.add_filters(filters)
+        if policies:
+            self.add_policies(policies)
+        if tests:
+            self.add_tests(tests)
+        if extensions:
+            self.add_extensions(*extensions)
+
 
 class SignerExtension(Extension):
     def __init__(self, secret_key: str) -> None:
@@ -277,3 +342,14 @@ class SignerExtension(Extension):
             return True, self.timed_unsign(signed_value, max_age)
         except itsdangerous.BadSignature:
             return False, None
+
+
+class ConsoleExtension:
+    def __init__(self, commands: list[click.Command] = None) -> None:
+        self.commands = commands or []
+
+    def add(self, *command: click.Command) -> None:
+        self.commands.extend(command)
+
+    def __iter__(self) -> typing.Iterator[click.Command]:
+        return iter(self.commands)
